#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid`` -- Rattail's Pyramid Framework
"""

from rattail.pyramid._version import __version__

from sqlalchemy.orm import sessionmaker, scoped_session


Session = scoped_session(sessionmaker())


def includeme(config):

    import rattail
    import rattail.db
    from zope.sqlalchemy import ZopeTransactionExtension

    # Configure Beaker session.
    config.include('pyramid_beaker')

    # Bring in transaction manager.
    config.include('pyramid_tm')

    # Configure SQLAlchemy session.
    rattail.init_modules(['rattail.db'])
    Session.configure(bind=rattail.db.engine)
    Session.configure(extension=ZopeTransactionExtension())

    # Configure user authentication / authorization.
    from pyramid.authentication import SessionAuthenticationPolicy
    config.set_authentication_policy(SessionAuthenticationPolicy())
    from rattail.pyramid.auth import RattailAuthorizationPolicy
    config.set_authorization_policy(RattailAuthorizationPolicy())

    # Add forbidden view.
    config.add_forbidden_view('rattail.pyramid.views.core:forbidden')

    # Add static views.
    config.add_static_view('rattail', 'rattail.pyramid:static')

    # Add subscriber hooks.
    config.include('rattail.pyramid.subscribers')
    # config.include('rattail.pyramid.views')
